#!/usr/bin/env python

"""
``dtail.views.reports`` -- Report Views
"""

import os
import os.path
import re

from mako.template import Template

from pyramid.response import Response

import rattail
from rattail.pyramid import Session
from rattail.db.model import Department, Brand, Vendor, Product, ProductCost
from rattail.files import resource_path
from rattail.time import local_time
from rattail.enum import UNIT_OF_MEASURE_POUND


plu_upc_pattern = re.compile(r'^000000000(\d{5})$')
weighted_upc_pattern = re.compile(r'^002(\d{5})00000\d$')

def get_upc(product):
    upc = '%014u' % product.upc
    m = plu_upc_pattern.match(upc)
    if m:
        return str(int(m.group(1)))
    m = weighted_upc_pattern.match(upc)
    if m:
        return str(int(m.group(1)))
    return upc


def inventory_report(request):
    """
    This is the "Inventory Worksheet" report.
    """

    departments = Session.query(Department)

    if request.params.get('department'):
        department = departments.get(request.params['department'])
        if department:
            body = write_inventory_worksheet(request, department)
            response = Response(content_type='text/html')
            response.headers['Content-Length'] = len(body)
            response.headers['Content-Disposition'] = 'attachment; filename=inventory.html'
            response.text = body
            return response

    departments = departments.order_by(Department.name)
    departments = departments.all()
    return{'departments': departments}


def write_inventory_worksheet(request, department):
    """
    Generates the Inventory Worksheet report.
    """

    def get_products(subdepartment):
        q = Session.query(Product)
        q = q.outerjoin(Brand)
        q = q.filter(Product.subdepartment == subdepartment)
        if request.params.get('weighted-only'):
            q = q.filter(Product.unit_of_measure == UNIT_OF_MEASURE_POUND)
        q = q.order_by(Brand.name, Product.description)
        return q.all()

    now = local_time()
    data = dict(
        date=now.strftime('%a %d %b %Y'),
        time=now.strftime('%I:%M %p'),
        department=department,
        get_products=get_products,
        get_upc=get_upc,
        )

    report = resource_path('rattail.pyramid:reports/inventory_worksheet.mako')
    template = Template(filename=report)
    return template.render(**data)
    

def ordering_report(request):
    """
    This is the "Ordering Worksheet" report.
    """

    if request.params.get('vendor'):
        vendor = Session.query(Vendor).get(request.params['vendor'])
        if vendor:
            departments = []
            uuids = request.params.get('departments')
            if uuids:
                for uuid in uuids.split(','):
                    dept = Session.query(Department).get(uuid)
                    if dept:
                        departments.append(dept)
            body = write_ordering_worksheet(vendor, departments)
            response = Response(content_type='text/html')
            response.headers['Content-Length'] = len(body)
            response.headers['Content-Disposition'] = 'attachment; filename=ordering.html'
            response.text = body
            return response
    return {}


def write_ordering_worksheet(vendor, departments):
    """
    Rendering engine for the ordering worksheet report.
    """

    q = Session.query(ProductCost)
    q = q.join(Product)
    q = q.filter(ProductCost.vendor == vendor)
    q = q.filter(Product.department_uuid.in_([x.uuid for x in departments]))

    costs = {}
    for cost in q:
        dept = cost.product.department
        subdept = cost.product.subdepartment
        costs.setdefault(dept, {})
        costs[dept].setdefault(subdept, [])
        costs[dept][subdept].append(cost)

    now = local_time()
    data = dict(
        vendor=vendor,
        costs=costs,
        date=now.strftime('%a %d %b %Y'),
        time=now.strftime('%I:%M %p'),
        get_upc=get_upc,
        rattail=rattail,
        )

    report = resource_path('rattail.pyramid:reports/ordering_worksheet.mako')
    template = Template(filename=report)
    return template.render(**data)


def includeme(config):

    config.add_route('reports.inventory', '/reports/inventory')
    config.add_view(inventory_report,
                    route_name='reports.inventory',
                    renderer='/reports/inventory.mako')

    config.add_route('reports.ordering', '/reports/ordering')
    config.add_view(ordering_report,
                    route_name='reports.ordering',
                    renderer='/reports/ordering.mako')
